import { BehaviorSubject } from "rxjs/BehaviorSubject";
import { Observable } from "rxjs/Observable";
import { DataObserverOptions } from "../interfaces";
/**
 * This class is used to create a observable from a provided observable source or created a new observable using `BehaviousSubject`
 * @class
 */
export declare class DataObserver<T> {
    options: DataObserverOptions<T>;
    /** Inbuild observable */
    dataBehavious$: BehaviorSubject<T | any | null>;
    entityName: string;
    initialData: {};
    /**
     * Provided data source
     */
    dataSource$: Observable<T>;
    /**
     *
     * @param {DataObserverOptions<T>} options - Options for data Observable
     */
    constructor(options?: DataObserverOptions<T>);
    setOptions(options?: DataObserverOptions<T>): void;
    /**
     *
     * @param dataSource$   Sets dataSource if not null
     *
     */
    setDataSource(dataSource$: any): void;
    /**
     *
     * @param initialData Sets initial Data if not null
     */
    setInitialData(initialData: any): void;
    /**
     *
     * @returns {Observable<T> | Observable<any | any | T>} Return Inbuild data source or provided data source
     */
    data$(): Observable<any>;
    onDataUpdate(data: T): void;
    /**
     * Updated data of inbuilt data source, would not work for provided data source
     * @param {T} data
     */
    updateData(data: T): void;
    getDataBehaviour(): BehaviorSubject<any>;
}
