import { Observable } from "rxjs/Observable";
import { IDateRange } from "../interfaces";
export declare abstract class EntityClient {
    /**
     * API class which handles requests for the entity
     */
    api: any;
    /**
     * Appended at start of date range params. Example if start date of date range is x the params in query will be
     * min_<dateRangeParam> like min_created_at
     * @type {string}
     */
    dateRangeParam: string;
    /**
     * Returns entity array or if selected just the selected entity id as array of length 1
     * @param id$
     * @param dataArray$
     * @param selected$
     * @returns {any}
     */
    dataArrayWithSelected$(id$: any, dataArray$: any, selected$: any): Observable<any>;
    pageDataWithSelected$(id$: any, pageData$: any, selected$: any): Observable<{
        results: any;
        count: number;
        next: string;
        previous: string;
    }>;
    clearData(): void;
    /**
     * Converts date range to query params object
     * @param {IDateRange} range
     * @returns {Object}
     */
    getQueryFromDateRange(range: IDateRange): object;
    getPageFromEntity(item$: any): any;
}
