import { HtUsersIndexClient } from "./users-index-client";
import { HtUserPlacelineClient } from "./user-placeline-client";
import { AllData, IDateRange, IUsersClientOptions, QueryLabel } from "../../interfaces";
import { Partial, IUserData, IUser, IUserAnalytics } from "ht-models";
import { HtUsersAnalytics } from "./users-analytics-client";
import { Observable } from "rxjs/Observable";
import { EntityClient } from "../../base/entity-client";
import { HtUsersAnalyticsMarkers } from "./users-analytics-markers";
import { ApiType } from "../../interfaces";
import { HtUsersIndexMarkers } from "./users-index-markers";
import { DefaultUsersFilter } from "../../filters/users-filter";
import { QueryObserver } from "../../base/query-observer";
import { HtMapClass } from "ht-js-map";
import { Store } from "../../store/store";
import * as fromRoot from "../../reducers";
import * as fromUsers from "../../reducers/user-reducer";
import * as fromSegments from "../../reducers/segments-reducer";
import * as fromUsersDispatcher from "../../dispatchers/user-dispatcher";
import * as fromQueryDispatcher from "../../dispatchers/query-dispatcher";
import { HtUsersSummaryClient } from "./users-summary-client";
/**
 * Class containing all user related client entity like list of user, user placeline etc
 * @class
 */
export declare class HtUsersClient extends EntityClient {
    private store;
    options: IUsersClientOptions;
    /**
     * Fetches from `/users/`
     */
    index: HtUsersIndexClient;
    /**
     * Fetches from `/users/analytics/`
     */
    analytics: HtUsersAnalytics;
    /**
     * Fetches user placeline
     */
    placeline: HtUserPlacelineClient;
    /**
     * Fetches across the page (complete data of users) from `/users/analytics/`
     */
    marksAnalytics: HtUsersAnalyticsMarkers;
    /**
     * Fetches across the page (complete data of users) from `/users/`
     */
    marksIndex: HtUsersIndexMarkers;
    filterClass: DefaultUsersFilter;
    dateRangeObserver: QueryObserver;
    initialDateRange: IDateRange;
    mapClass: HtMapClass;
    userDispatcher: typeof fromUsersDispatcher;
    queryDispatcher: typeof fromQueryDispatcher;
    list: any;
    summary: HtUsersSummaryClient;
    markers: any;
    _statusQueryArray: QueryLabel[];
    constructor(req: any, store: Store<fromRoot.State>, options?: IUsersClientOptions);
    statusQueryArray: QueryLabel[];
    setActive(): void;
    getInitialDateRange(range?: Partial<IDateRange>): IDateRange;
    /**
     * Return users list array or array of length 1 with selected user if user is selected (userId not null)
     * @returns {any}
     */
    placelineOrList$(): Observable<any>;
    listPage$(): Observable<{
        results: any;
        count: number;
        next: string;
        previous: string;
    }>;
    listSummary$(): Observable<any>;
    listStatusOverview$(): Observable<{
        location_disabled: number;
        logged_off: number;
        at_place: number;
        on_trip: number;
        network_offline: number;
    }>;
    listStatusChart$(): Observable<{
        totalUsers: number;
        chart: any;
        hasSelected: boolean;
    }>;
    listMap$(): Observable<{
        placelineList: any;
        summary: any;
    }>;
    /**
     * Return label of current user queries
     * @returns {Observable<QueryLabel[]>}
     */
    readonly queryLabel$: any;
    /**
     * Return orderings labels and sign for display
     * @returns {Observable<any>}
     */
    readonly ordering$: any;
    /**
     * Return display string for date range
     * @returns {Observable<string>}
     */
    readonly dateRangeDisplay$: Observable<string>;
    /**
     * Processes ordering text to extract string and sign
     * @param {string} ordering
     * @returns {{string: string; sign: number}}
     */
    private getOrderingMod(ordering);
    getListClient(apiType: ApiType): HtUsersIndexClient | HtUsersAnalytics;
    getMarkerClient(apiType: ApiType): HtUsersAnalyticsMarkers | HtUsersIndexMarkers;
    /**
     * Return array of markers to display. Return [] for selected placeline
     * @returns {any}
     */
    allMarkers$(): Observable<any>;
    getState(): Store<fromUsers.State>;
    getPlacelineQuery(): Observable<object>;
    getUsersIndexPage(): Observable<any>;
    getUsersAnalyticsPage(): Observable<any>;
    getUsersListPage(): Observable<any>;
    fromApiType(obIndex: any, obAnalytics: any): Observable<{}>;
    getUserListQuery(): Observable<object>;
    getUserListApiType(): Store<ApiType>;
    getSegmentsStates(): Store<fromSegments.State>;
    setUserData(userData: IUserData): void;
    setUsersIndexPage(usersPage: any): void;
    setUsersAnalyticsPage(usersPage: any): void;
    setIndexMarkers(data: AllData<IUser>): void;
    setAnalyticsMarkers(data: AllData<IUserAnalytics>): void;
    setLoadingUserData(data: any): void;
    setLoadingUserAnalytics(data: any): void;
    setLoadingUserIndex(data: any): void;
    setLoadingUserAnalyticsAll(data: any): void;
    setLoadingUserIndexAll(data: any): void;
    private initEffects();
}
