import { ActionReducerMap } from "../store/models";
import { Action, ActionReducer } from "../store/models";
import { MemoizedSelector } from "../store/selector";
import * as fromUsers from "./user-reducer";
import * as fromSegments from "./segments-reducer";
import * as fromQuery from "./query-reducer";
import * as fromLoading from "./loading-reducer";
import * as fromGroups from "./groups-reducer";
import { IUserData, IUser, IUserAnalytics, IGroup, IUserListSummary } from "ht-models";
import { ApiType, AllData } from "../interfaces";
export interface State {
    users: fromUsers.State;
    segments: fromSegments.State;
    query: fromQuery.State;
    loading: fromLoading.State;
    groups: fromGroups.State;
}
export declare const reducers: ActionReducerMap<State>;
export declare function reducer(state: any, action: any): ActionReducer<State, Action>;
export declare const metaReducers: ActionReducer<any, any>[];
/**
 * Users selectors
 */
export declare const getUsersState: MemoizedSelector<object, fromUsers.State>;
export declare const getUsersUsersData: MemoizedSelector<object, IUserData>;
export declare const getUsersIndexPage: MemoizedSelector<object, {
    count: number;
    next: string;
    previous: string;
    results: IUser[];
}>;
export declare const getUsersAnalyticsPage: MemoizedSelector<object, {
    count: number;
    next: string;
    previous: string;
    results: IUserAnalytics[];
}>;
export declare const getUsersListApiType: MemoizedSelector<object, ApiType>;
export declare const getUsersListActive: MemoizedSelector<object, boolean>;
export declare const getUsersIndexIsActive: MemoizedSelector<object, boolean>;
export declare const getUsersAnalyticsIsActive: MemoizedSelector<object, boolean>;
export declare const getUsersIndexMarkersIsActive: MemoizedSelector<object, boolean>;
export declare const getUsersAnalyticsMarkersIsActive: MemoizedSelector<object, boolean>;
export declare const getUsersIndexFilteredMarker: MemoizedSelector<object, any>;
export declare const getUsersAnalyticsFilteredMarker: MemoizedSelector<object, any>;
export declare const getUsersIndexAll: MemoizedSelector<object, AllData<IUser>>;
export declare const getUsersAnalyticsAll: MemoizedSelector<object, AllData<IUserAnalytics>>;
export declare const getUsersSummary: MemoizedSelector<object, IUserListSummary>;
export declare const getUsersSummaryActive: MemoizedSelector<object, boolean>;
/**
 * Segment selectors
 */
export declare const getSegmentsState: MemoizedSelector<object, fromSegments.State>;
export declare const getSegmentsSelectedId: MemoizedSelector<object, string>;
export declare const getSegmentsResetMapId: MemoizedSelector<object, string>;
/**
 * Query selectors
 */
export declare const getQueryState: MemoizedSelector<object, fromQuery.State>;
export declare const getQueryPlacelineId: MemoizedSelector<object, string>;
export declare const getQueryUserQuery: MemoizedSelector<object, object>;
export declare const getQueryUserId: MemoizedSelector<object, string>;
/**
 *
 * Loading selectors.
 */
export declare const getLoadingState: MemoizedSelector<object, fromLoading.State>;
export declare const getLoadingUserData: MemoizedSelector<object, boolean>;
export declare const getLoadingAnalytics: MemoizedSelector<object, boolean>;
export declare const getLoadingUserIndex: MemoizedSelector<object, boolean>;
export declare const getLoadingUserAnalyticsAll: MemoizedSelector<object, boolean>;
export declare const getLoadingUserIndexAll: MemoizedSelector<object, boolean>;
export declare const getLoadingUserSummary: MemoizedSelector<object, boolean>;
/**
 *
 * Groups reducer
 */
export declare const getGroupsState: MemoizedSelector<object, fromGroups.State>;
export declare const getGroupId: MemoizedSelector<object, string>;
export declare const getGroupAll: MemoizedSelector<object, AllData<IGroup>>;
export declare const getGroupListActive: MemoizedSelector<object, boolean>;
